import QtQuick 2.0
import Sailfish.Silica 1.0
import Sailfish.Lipstick 1.0
import org.nemomobile.configuration 1.0
import "devices.js" as Devices

Page {
    ConfigurationGroup {
        id: currentSettings
        path: "/apps/patchmanager/patch-powerdrain"
        property bool enabled: true
        property bool vibrate: false
        property int warn: 400
        property int critoffset: 250
        property bool show: false
        property int interval: 7000
        property string currentpath: "/sys/class/power_supply/battery/current_now"
        property int classidx: 0
    }

    SilicaFlickable {
        anchors.fill: parent
        contentHeight: content.height

        Column {
            id: content
            width: parent.width
            spacing: 0

            PageHeader {
                title: "Power drain indicator"
                //description: "Operation settings"
            }

            TextSwitch {
                id: enableSwitch
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Enabled"
                description: "The battery icon will change color if battery drain reaches warning or critical levels."
                checked: currentSettings.enabled
                onCheckedChanged: currentSettings.enabled = checked
            }
            SectionHeader { text: "Preview"}
            SilicaItem {
                // as the indicator animates itself, don't extend to the edges if its shown.
                width: bindi.visible ? bindi.width : parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                height: Theme.itemSizeLarge
                property bool showPreview: enableSwitch.checked
                BatteryStatusIndicator { id: bindi; visible: parent.showPreview } // it internally assumes its's left-aligned
                Label {
                    visible: !parent.showPreview
                    anchors.centerIn: parent
                    width: parent.width - Theme.horizontalPageMargin * 2
                    horizontalAlignment: Text.AlignHCenter
                    wrapMode: Text.WordWrap
                    color: Theme.secondaryColor
                    text: "Enable the indicator first."
                    font.pixelSize: Theme.fontSizeExtraSmall
                }
            }
            Label {
                width: parent.width - Theme.horizontalPageMargin*2
                anchors.horizontalCenter: parent.horizontalCenter
                wrapMode: Text.WordWrap
                color: Theme.highlightColor
                text: "Note about measurements: \n"
                     + "The indicator will show the average of exactly 5 samples, taken at the interval specified.\n"
                     + "Measurements will be taken only while the indicator is visible, a charger is not connected, and Power Saving Mode is inactive. \n"
                     + "This means values displayed immediately after it became visible will be skewed. \n"
                     + "Use other tools if you need precise and accurate monitoring."
                font.pixelSize: Theme.fontSizeSmall
            }
            SectionHeader { text: "Update interval"}
            Label {
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignRight
                wrapMode: Text.WordWrap
                color: Theme.secondaryColor
                text: "Lower values give greater accuracy, but also use more resources, possibly skewing the results. (This interval will be quadrupled when the charger is connected. Just Because.)"
                font.pixelSize: Theme.fontSizeExtraSmall
            }
            Slider {
                id: intervalSlider
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter

                minimumValue: 0.5
                maximumValue: 15
                stepSize: 0.5
                value: currentSettings.interval / 1000

                valueText: value + "s"

                onDownChanged: if (!down) { currentSettings.interval = sliderValue * 1000 }
            }
            TextSwitch {
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Show Drain"
                description: "In addition to color indications show the current current next to the battery gauge."
                checked: currentSettings.show
                onCheckedChanged: currentSettings.show = checked
            }
            SectionHeader { text: "Warning"}
            Label { 
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignRight
                wrapMode: Text.WordWrap
                color: Theme.secondaryColor
                text: "Amount of current that is treated as warning level."
                font.pixelSize: Theme.fontSizeExtraSmall
            }
            Slider {
                id: warnSlider
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter

                minimumValue: 100
                maximumValue: 1000
                stepSize: 50
                value: currentSettings.warn

                valueText: value + " mA"

                onDownChanged: if (!down) { currentSettings.warn = sliderValue }
            }
            SectionHeader { text: "Critical"}
            Label { 
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignRight
                wrapMode: Text.WordWrap
                color: Theme.secondaryColor
                text: "Offset from the warning level that is treated as critical."
                font.pixelSize: Theme.fontSizeExtraSmall
            }
            Slider {
                id: critSlider
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter

                minimumValue: 50
                maximumValue: 500
                stepSize: 50
                value: currentSettings.critoffset

                valueText: "+" + value + " mA"
                label: "treshold: " + (warnSlider.sliderValue + sliderValue) + " mA"

                onDownChanged:  { if (!down) { currentSettings.critoffset = Math.max(sliderValue, sliderValue + minimumValue) } }
            }
            TextSwitch {
                id: vibrateSwitch
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                text: "Vibrate on critical"
                description: "Vibrate when battery drain reaches critical levels."
                checked: currentSettings.vibrate
                onCheckedChanged: currentSettings.vibrate = checked
            }
            SectionHeader { text: "Data Source"}
            Label {
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                horizontalAlignment: Text.AlignRight
                wrapMode: Text.WordWrap
                color: Theme.secondaryColor
                text: "Where the current is read from."
                font.pixelSize: Theme.fontSizeExtraSmall
            }
            ComboBox {
                width: parent.width - Theme.horizontalPageMargin * 2
                anchors.horizontalCenter: parent.horizontalCenter
                label: "Device Class:"
                value: currentItem.text
                currentIndex: currentSettings.classidx
                description: "Path: " + currentItem.currentPath
                onValueChanged: {
                    if (value) { currentSettings.currentpath = currentItem.currentPath }
                    if (currentIndex >= 0 ) { currentSettings.classidx = currentIndex }
                }
                menu: ContextMenu {
                    Repeater {
                        model: ListModel {
                            id: classModel
                            //ListElement { name: "Test1"; path: "/sys/class/power_supply/battery/current_now" }
                            //ListElement { name: "Test2"   ; path: "/sys/class/power_supply/battery/BAT0/current" }
                            Component.onCompleted: {
                                var d = Devices.devices;
                                for (var i=0; i<d.length; i++) {
                                    classModel.append(d[i]);
                                }
                            }
                        }
                        delegate: MenuItem { text: name; property string currentPath: path }
                    }
                }
            }
        }
    }
}
